@@ -6,7 +6,6 @@
 #include <string.h>
 
 #include <atlbase.h>
-#include <atlconv.h>
 #include <comdef.h>
 #include <windows.h>
 #include <wia.h>
@@ -15,7 +14,6 @@
 #include <Python.h>
 
 #include "properties.h"
-#include "trace.h"
 #include "transfer.h"
 #include "util.h"
 
@@ -49,17 +47,9 @@ static PyObject *init(PyObject *, PyObject* args)
         return NULL;
     }
 
-    if (!PyEval_ThreadsInitialized()) {
-        wia_log(WIA_WARNING, "Python thread not yet initialized ?!");
-        PyEval_InitThreads();
-    }
-
-    wia_log(WIA_INFO, "WIA->init()");
-
     hr = CoInitialize(NULL);
     if (FAILED(hr)) {
-        wia_log(WIA_WARNING, "CoInitialize() failed !");
-        wia_log_hresult(WIA_WARNING, hr);
+        WIA_WARNING("Pyinsane: WARNING: CoInitialize() failed !");
         Py_RETURN_NONE;
     }
 
@@ -86,8 +76,7 @@ static HRESULT get_device_basic_infos(IWiaPropertyStorage *properties,
 
     hr = properties->ReadMultiple(3 /* nb_properties */, input, output);
     if (FAILED(hr)) {
-        wia_log(WIA_WARNING, "WiaPropertyStorage->ReadMultiple() failed");
-        wia_log_hresult(WIA_WARNING, hr);
+        WIA_WARNING("Pyinsane: WiaPropertyStorage->ReadMultiple() failed");
         return hr;
     }
 
@@ -96,10 +85,6 @@ static HRESULT get_device_basic_infos(IWiaPropertyStorage *properties,
     assert(output[2].vt == VT_I4);
 
     if (GET_STIDEVICE_TYPE(output[2].lVal) != StiDeviceTypeScanner) {
-        CW2A dev_id_str(output[0].bstrVal);
-        CW2A dev_name_str(output[0].bstrVal);
-        wia_log(WIA_INFO, "[%s, %s] --> not a scanner --> ignored",
-                (char *)dev_id_str, (char *)dev_name_str);
         *out_tuple = NULL;
         return S_OK;
     }
@@ -121,7 +106,6 @@ static PyObject *get_devices(PyObject *, PyObject* args)
     CComPtr<IWiaDevMgr2> wia_dev_manager;
     CComPtr<IEnumWIA_DEV_INFO> wia_dev_info_enum;
     unsigned long nb_devices;
-    int actual_nb_devices = 0;
     PyObject *dev_infos;
     PyObject *all_devs;
 
@@ -129,20 +113,16 @@ static PyObject *get_devices(PyObject *, PyObject* args)
         return NULL;
     }
 
-    wia_log(WIA_INFO, "WIA->get_devices()");
-
     // Create a connection to the local WIA device manager
     hr = wia_dev_manager.CoCreateInstance(CLSID_WiaDevMgr2);
     if (FAILED(hr)) {
-        wia_log(WIA_WARNING, "CoCreateInstance failed");
-        wia_log_hresult(WIA_WARNING, hr);
+        WIA_WARNING("Pyinsane: WARNING: CoCreateInstance failed");
         Py_RETURN_NONE;
     }
 
     hr = wia_dev_manager->EnumDeviceInfo(WIA_DEVINFO_ENUM_ALL, &wia_dev_info_enum);
     if (FAILED(hr)) {
-        wia_log(WIA_WARNING, "WiaDevMgr->EnumDeviceInfo() failed");
-        wia_log_hresult(WIA_WARNING, hr);
+        WIA_WARNING("Pyinsane: WARNING: WiaDevMgr->EnumDeviceInfo() failed");
         Py_RETURN_NONE;
     }
 
@@ -150,8 +130,7 @@ static PyObject *get_devices(PyObject *, PyObject* args)
 
     hr = wia_dev_info_enum->GetCount(&nb_devices);
     if (FAILED(hr)) {
-        wia_log(WIA_WARNING, "GetCount() failed !");
-        wia_log_hresult(WIA_WARNING, hr);
+        WIA_WARNING("PyInsane: WARNING: GetCount() failed !");
         Py_RETURN_NONE;
     }
 
@@ -171,15 +150,12 @@ static PyObject *get_devices(PyObject *, PyObject* args)
             // not a scanner
             continue;
         }
-        actual_nb_devices++;
 
         properties->Release();
 
         PyList_Append(all_devs, dev_infos);
     }
 
-    wia_log(WIA_DEBUG, "%d devices found", actual_nb_devices);
-
     // wia_dev_info_enum->Release(); // TODO(Jflesch) ?
     return all_devs;
 }
@@ -206,29 +182,23 @@ static PyObject *open_device(PyObject *, PyObject *args)
         return NULL;
     }
 
-    wia_log(WIA_INFO, "Opening device(%s)", devid);
-
     hr = wia_dev_manager.CoCreateInstance(CLSID_WiaDevMgr2);
     if (FAILED(hr)) {
-        wia_log(WIA_WARNING, "CoCreateInstance failed");
-        wia_log_hresult(WIA_WARNING, hr);
+        WIA_WARNING("Pyinsane: WARNING: CoCreateInstance failed");
         Py_RETURN_NONE;
     }
 
     dev = (struct wia_device *)calloc(1, sizeof(struct wia_device));
     dev->dev_manager = wia_dev_manager;
 
-    bstr_devid = SysAllocString(A2W(devid));
+    bstr_devid = SysAllocString(A2W(devid)); // TODO(Jflesch): Does any of this allocate anything ? oO
     hr = wia_dev_manager->CreateDevice(0, bstr_devid, &dev->device);
     if (FAILED(hr)) {
-        wia_log(WIA_WARNING, "WiaDevMgr->CreateDevice() failed");
-        wia_log_hresult(WIA_WARNING, hr);
+        WIA_WARNING("Pyinsane: WARNING: WiaDevMgr->CreateDevice() failed");
         free(dev);
         Py_RETURN_NONE;
     }
 
-    wia_log(WIA_INFO, "Device(%s) opened", devid);
-
     return PyCapsule_New(dev, WIA_PYCAPSULE_DEV_NAME, free_device);
 }
 
@@ -250,7 +220,6 @@ static PyObject *get_sources(PyObject *, PyObject *args)
     PyObject *capsule;
     PyObject *tuple;
     PyObject *all_sources;
-    int nb_sources = 0;
     struct wia_source *source;
     PROPSPEC input[2] = {0};
     PROPVARIANT output[2] = {0};
@@ -261,19 +230,17 @@ static PyObject *get_sources(PyObject *, PyObject *args)
     input[1].ulKind = PRSPEC_PROPID;
     input[1].propid = WIA_IPA_ITEM_CATEGORY;
 
-    wia_log(WIA_INFO, "Getting sources");
-
     if (!PyArg_ParseTuple(args, "O", &capsule)) {
-        wia_log(WIA_WARNING, "get_sources(): Invalid args");
+        WIA_WARNING("Pyinsane: get_sources(): Invalid args");
         return NULL;
     }
     if (!PyCapsule_CheckExact(capsule)) {
-        wia_log(WIA_WARNING, "get_sources(): invalid argument type (not a pycapsule)");
+        WIA_WARNING("Pyinsane: WARNING: get_sources(): invalid argument type (not a pycapsule)");
         Py_RETURN_NONE;
     }
 
     if ((dev = (struct wia_device *)PyCapsule_GetPointer(capsule, WIA_PYCAPSULE_DEV_NAME)) == NULL) {
-        wia_log(WIA_WARNING, "get_sources(): invalid argument type");
+        WIA_WARNING("Pyinsane: WARNING: get_sources(): invalid argument type");
         Py_RETURN_NONE;
     }
 
@@ -283,8 +250,6 @@ static PyObject *get_sources(PyObject *, PyObject *args)
     while(hr == S_OK) {
         hr = enum_item->Next(1, &child, NULL);
         if (hr != S_OK) {
-            wia_log(WIA_WARNING, "get_sources(): enum_item->next() failed");
-            wia_log_hresult(WIA_WARNING, hr);
             continue;
         }
 
@@ -296,8 +261,7 @@ static PyObject *get_sources(PyObject *, PyObject *args)
 
         hr = child_properties->ReadMultiple(2 /* nb_properties */, input, output);
         if (FAILED(hr)) {
-            wia_log(WIA_WARNING, "WiaPropertyStorage->ReadMultiple() failed");
-            wia_log_hresult(WIA_WARNING, hr);
+            WIA_WARNING("Pyinsane: WiaPropertyStorage->ReadMultiple() failed");
             child->Release();
             continue;
         }
@@ -305,39 +269,29 @@ static PyObject *get_sources(PyObject *, PyObject *args)
         assert(output[0].vt == VT_BSTR);
         assert(output[1].vt == VT_CLSID);
 
-        CW2A source_name_str(output[0].bstrVal);
-
         if (*output[1].puuid == WIA_CATEGORY_FINISHED_FILE
                     || *output[1].puuid == WIA_CATEGORY_FOLDER
                     || *output[1].puuid == WIA_CATEGORY_ROOT) {
-            wia_log(WIA_WARNING, "Ignoring unmanaged source type for [%s]", (char *)source_name_str);
-            free(source);
-            continue;
+                free(source);
+                continue;
         } else if (*output[1].puuid == WIA_CATEGORY_AUTO) {
-            source->type = WIA_SRC_AUTO;
-            wia_log(WIA_INFO, "Got source [%s] (auto)", (char *)source_name_str);
+                source->type = WIA_SRC_AUTO;
         } else if (*output[1].puuid == WIA_CATEGORY_FEEDER
-                || *output[1].puuid == WIA_CATEGORY_FEEDER_BACK
-                || *output[1].puuid == WIA_CATEGORY_FEEDER_FRONT) {
-            source->type = WIA_SRC_FEEDER;
-            wia_log(WIA_INFO, "Got source [%s] (feeder)", (char *)source_name_str);
+                    || *output[1].puuid == WIA_CATEGORY_FEEDER_BACK
+                    || *output[1].puuid == WIA_CATEGORY_FEEDER_FRONT) {
+                source->type = WIA_SRC_FEEDER;
         } else {
             source->type = WIA_SRC_FLATBED;
-            wia_log(WIA_INFO, "Got source [%s] (assuming flatbed)", (char *)source_name_str);
         }
 
-
         source_name = PyUnicode_FromWideChar(output[0].bstrVal, -1);
         capsule = PyCapsule_New(source, WIA_PYCAPSULE_SRC_NAME, free_source);
         tuple = PyTuple_Pack(2, source_name, capsule);
 
         PyList_Append(all_sources, tuple);
-        nb_sources++;
         Py_DECREF(tuple);
     }
 
-    wia_log(WIA_INFO, "%d sources found", nb_sources);
-
     return all_sources;
 }
 
@@ -348,7 +302,7 @@ static IWiaItem2 *capsule2item(PyObject *capsule)
     struct wia_source *wia_src;
 
     if (!PyCapsule_CheckExact(capsule)) {
-        wia_log(WIA_WARNING, "invalid argument type (not a pycapsule)");
+        WIA_WARNING("Pyinsane: WARNING: invalid argument type (not a pycapsule)");
         return NULL;
     }
 
@@ -364,7 +318,7 @@ static IWiaItem2 *capsule2item(PyObject *capsule)
             return wia_src->source;
     }
 
-    wia_log(WIA_WARNING, "Invalid argument type (not a known pycapsule type)");
+    WIA_WARNING("Pyinsane: WARNING: Invalid argument type (not a known pycapsule type)");
     return NULL;
 }
 
@@ -387,17 +341,15 @@ static PyObject *get_properties(PyObject *, PyObject *args)
     PyObject *access_right;
     PyObject *possible_values;
 
+
     if (!PyArg_ParseTuple(args, "O", &capsule)) {
-        wia_log(WIA_WARNING, "get_sources(): Invalid args");
+        WIA_WARNING("Pyinsane: WARNING: get_sources(): Invalid args");
         return NULL;
     }
 
-    wia_log(WIA_INFO, "Getting Properties");
-
     item = capsule2item(capsule);
-    if (item == NULL) {
+    if (item == NULL)
         Py_RETURN_NONE;
-    }
 
     for (nb_properties = 0 ; g_wia_all_properties[nb_properties].name != NULL ; nb_properties++)
     { }
@@ -413,8 +365,7 @@ static PyObject *get_properties(PyObject *, PyObject *args)
     CComQIPtr<IWiaPropertyStorage> properties(item);
     hr = properties->ReadMultiple(nb_properties, input, output);
     if (FAILED(hr)) {
-        wia_log(WIA_WARNING, "WiaPropertyStorage->ReadMultiple() failed");
-        wia_log_hresult(WIA_WARNING, hr);
+        WIA_WARNING("Pyinsane: WARNING: WiaPropertyStorage->ReadMultiple() failed");
         free(input);
         free(output);
         Py_RETURN_NONE;
@@ -429,7 +380,7 @@ static PyObject *get_properties(PyObject *, PyObject *args)
         if (output[i].vt == 0)
             continue;
         if (output[i].vt != g_wia_all_properties[i].vartype) {
-            wia_log(WIA_WARNING, "A property has a type different from the one expected");
+            WIA_WARNING("Pyinsane: WARNING: A property has a type different from the one expected");
             continue;
         }
         switch(output[i].vt) {
@@ -452,14 +403,12 @@ static PyObject *get_properties(PyObject *, PyObject *args)
                 propvalue = clsid_to_pyobject(&g_wia_all_properties[i], *output[i].puuid);
                 break;
             default:
-                wia_log(WIA_WARNING, "Unknown var type");
+                WIA_WARNING("Pyinsane: WARNING: Unknown var type");
                 assert(0);
                 continue;
         }
         if (propvalue == NULL)
             continue;
-
-        wia_log(WIA_DEBUG, "Got property [%s]", g_wia_all_properties[i].name);
         propname = PyUnicode_FromString(g_wia_all_properties[i].name);
 
         access_right = (g_wia_all_properties[i].rw ? rw : ro);
@@ -472,8 +421,6 @@ static PyObject *get_properties(PyObject *, PyObject *args)
         Py_DECREF(prop);
     }
 
-    wia_log(WIA_DEBUG, "Got all properties");
-
     free(output);
     return all_props;
 }
@@ -494,16 +441,13 @@ static PyObject *get_constraints(PyObject *, PyObject *args)
     PyObject *prop;
 
     if (!PyArg_ParseTuple(args, "O", &capsule)) {
-        wia_log(WIA_WARNING, "get_sources(): Invalid args");
+        WIA_WARNING("Pyinsane: WARNING: get_sources(): Invalid args");
         return NULL;
     }
 
-    wia_log(WIA_INFO, "Getting constraints");
-
     item = capsule2item(capsule);
-    if (item == NULL) {
+    if (item == NULL)
         Py_RETURN_NONE;
-    }
 
     for (nb_properties = 0 ; g_wia_all_properties[nb_properties].name != NULL ; nb_properties++)
     { }
@@ -520,8 +464,7 @@ static PyObject *get_constraints(PyObject *, PyObject *args)
     CComQIPtr<IWiaPropertyStorage> properties(item);
     hr = properties->GetPropertyAttributes(nb_properties, input, prop_attributes, output);
     if (FAILED(hr)) {
-        wia_log(WIA_WARNING, "WiaPropertyStorage->GetPropertyAttribute() failed. Will use defaults");
-        wia_log_hresult(WIA_WARNING, hr);
+        WIA_WARNING("Pyinsane: WARNING: WiaPropertyStorage->GetPropertyAttribute() failed. Will use defaults");
         free(input);
         free(prop_attributes);
         free(output);
@@ -550,18 +493,15 @@ static PyObject *get_constraints(PyObject *, PyObject *args)
                     constraint = int_vector_to_pyobject_list(&output[i].cal);
                 break;
             case VT_VECTOR | VT_UI2:
-                wia_log(WIA_WARNING, "Got VECTOR|UI2 as constraint. Not supported yet");
+                WIA_WARNING("Pyinsane: WARNING: Got VECTOR|UI2 as constraint. Not supported yet");
                 // TODO
                 continue;
             case VT_UI1 | VT_VECTOR:
-                wia_log(WIA_WARNING, "Got VECTOR|UI1 as constraint. Not supported yet");
+                WIA_WARNING("Pyinsane: WARNING: Got VECTOR|UI1 as constraint. Not supported yet");
                 // TODO
                 continue;
             case VT_BSTR:
-                constraint = PyUnicode_FromWideChar(
-                        SysAllocString(output[i].bstrVal),
-                        SysStringLen(output[i].bstrVal)
-                    );
+                constraint = PyUnicode_FromWideChar(SysAllocString(output[i].bstrVal), SysStringLen(output[i].bstrVal));
                 break;
             case VT_BSTR | VT_VECTOR:
                 constraint = str_vector_to_pyobject(&output[i].cabstr);
@@ -570,12 +510,11 @@ static PyObject *get_constraints(PyObject *, PyObject *args)
                 constraint = clsid_to_pyobject(&g_wia_all_properties[i], *output[i].puuid);
                 break;
             default:
-                wia_log(WIA_WARNING, "Unknown var type for constraint");
+                WIA_WARNING("Pyinsane: WARNING: Unknown var type for constraint");
                 continue;
         }
         if (constraint == NULL) {
-            wia_log(WIA_WARNING, "Failed to parse constraint of [%s]",
-                    g_wia_all_properties[i].name);
+            fprintf(stderr, "Pyinsane: Failed to parse constraint of [%s]\n", g_wia_all_properties[i].name);
             continue;
         }
         propname = PyUnicode_FromString(g_wia_all_properties[i].name);
@@ -584,8 +523,6 @@ static PyObject *get_constraints(PyObject *, PyObject *args)
         Py_DECREF(prop);
     }
 
-    wia_log(WIA_DEBUG, "Got all constraints");
-
     free(prop_attributes);
     free(output);
     return all_constraints;
@@ -603,13 +540,12 @@ static int _set_property(IWiaItem2 *item, const struct wia_property *property_sp
 
     PropVariantInit(&propvalue);
     propvalue.vt = property_spec->vartype;
-
+    
     switch(property_spec->vartype) {
         case VT_I4:
             propvalue.lVal = pyobject_to_int(property_spec, pyvalue, -1);
             if (propvalue.lVal == -1) {
-                wia_log(WIA_WARNING, "Setting property [%s]: pyobject_to_int() failed",
-                        property_spec->name);
+                WIA_WARNING("Pyinsane: pyobject_to_int() failed");
                 return 0;
             }
             break;
@@ -617,32 +553,27 @@ static int _set_property(IWiaItem2 *item, const struct wia_property *property_sp
             propvalue.vt = VT_I4;
             propvalue.lVal = pyobject_to_int(property_spec, pyvalue, -1);
             if (propvalue.lVal == -1) {
-                wia_log(WIA_WARNING, "Setting property [%s]: pyobject_to_int() failed",
-                        property_spec->name);
+                WIA_WARNING("Pyinsane: pyobject_to_int() failed");
                 return 0;
             }
             break;
         case VT_VECTOR | VT_UI2:
         case VT_UI1 | VT_VECTOR:
-            wia_log(WIA_WARNING, "Setting property [%s]: Vector not supported yet",
-                    property_spec->name);
+            WIA_WARNING("Pyinsane: WARNING: Vector not supported yet");
             // TODO
             return 0;
         case VT_BSTR:
-            wia_log(WIA_WARNING, "Setting property [%s]: String not supported yet",
-                    property_spec->name);
+            WIA_WARNING("Pyinsane: WARNING: String not supported yet");
             // TODO
             return 0;
         case VT_CLSID:
             if (!pyobject_to_clsid(property_spec, pyvalue, &propvalue.puuid)) {
-                wia_log(WIA_WARNING, "Setting property [%s]: pyobject_to_clsid() failed",
-                        property_spec->name);
+                WIA_WARNING("Pyinsane: pyobject_to_clsid() failed");
                 return 0;
             }
             break;
         default:
-            wia_log(WIA_WARNING, "Setting property [%s]: Unknown var type",
-                    property_spec->name);
+            WIA_WARNING("Pyinsane: WARNING: Unknown var type");
             assert(0);
             return 0;
     }
@@ -650,9 +581,8 @@ static int _set_property(IWiaItem2 *item, const struct wia_property *property_sp
     CComQIPtr<IWiaPropertyStorage> properties(item);
     hr = properties->WriteMultiple(1, &propspec, &propvalue, WIA_IPA_FIRST);
     if (FAILED(hr)) {
-        wia_log(WIA_WARNING, "Setting property [%s]: properties->WriteMultiple() failed",
-                property_spec->name);
-        wia_log_hresult(WIA_WARNING, hr);
+        WIA_WARNING("Pyinsane: WARNING: properties->WriteMultiple() failed");
+        fprintf(stderr, "Pyinsane: WARNING: properties->WriteMultiple() failed: %s : 0x%X\n", property_spec->name, hr);
         return 0;
     }
 
@@ -669,7 +599,7 @@ static PyObject *set_property(PyObject *, PyObject *args)
     int i;
 
     if (!PyArg_ParseTuple(args, "OOO", &capsule, &py_propname, &py_propvalue)) {
-        wia_log(WIA_WARNING, "get_sources(): Invalid args");
+        WIA_WARNING("Pyinsane: WARNING: get_sources(): Invalid args");
         return NULL;
     }
     item = capsule2item(capsule);
@@ -677,8 +607,6 @@ static PyObject *set_property(PyObject *, PyObject *args)
         Py_RETURN_FALSE;
 
     propname = PyUnicode_AsUTF8(py_propname);
-    wia_log(WIA_INFO, "Setting value of property [%s]", propname);
-
     for (i = 0 ; g_wia_all_properties[i].name != NULL ; i++) {
         if (strcmp(g_wia_all_properties[i].name, propname) != 0)
             continue;
@@ -688,7 +616,7 @@ static PyObject *set_property(PyObject *, PyObject *args)
         Py_RETURN_TRUE;
     }
 
-    wia_log(WIA_WARNING, "Pyinsame: set_property(): Property not found");
+    WIA_WARNING("Pyinsame: WARNING: set_property(): Property not found");
     Py_RETURN_FALSE;
 }
 
@@ -703,7 +631,6 @@ static void end_scan(PyObject *capsule)
 {
     struct wia_scan *scan;
 
-    wia_log(WIA_DEBUG, "End of scan");
     scan = (struct wia_scan *)PyCapsule_GetPointer(capsule, WIA_PYCAPSULE_SCAN_NAME);
     if (scan == NULL)
         return;
@@ -731,12 +658,9 @@ static void get_data_wrapper(const void *data, int nb_bytes, void *cb_data)
     PyObject *arglist;
     PyObject *res;
 
-    wia_log(WIA_DEBUG, "get_data_wrapper()");
-
     WaitForSingleObject(download->mutex, 0);
 
     PyEval_RestoreThread(download->_save);
-    wia_log_set_pythread_state(download->mutex, NULL);
 
     for ( ; nb_bytes > 0 ; nb_bytes -= nb, cdata += nb) {
         nb = (int)WIA_MIN(nb_bytes, download->buffer.len);
@@ -745,12 +669,11 @@ static void get_data_wrapper(const void *data, int nb_bytes, void *cb_data)
         arglist = Py_BuildValue("(i)", nb);
         res = PyEval_CallObject(download->get_data_cb, arglist);
         if (res == NULL) {
-            wia_log(WIA_WARNING, "Got exception from callback download->get_data_cb !");
+            WIA_WARNING("Pyinsane: WARNING: Got exception from callback download->get_data_cb !");
         }
         Py_XDECREF(res);
     }
 
-    wia_log_set_pythread_state(download->mutex, &download->_save);
     download->_save = PyEval_SaveThread();
 
     ReleaseMutex(download->mutex);
@@ -762,20 +685,14 @@ static void end_of_page_wrapper(void *cb_data)
     struct download *download = (struct download *)cb_data;
     PyObject *res;
 
-    wia_log(WIA_DEBUG, "end_of_page_wrapper()");
-
     WaitForSingleObject(download->mutex, 0);
 
     PyEval_RestoreThread(download->_save);
-    wia_log_set_pythread_state(download->mutex, NULL);
-
     res = PyEval_CallObject(download->end_of_page_cb, NULL);
     if (res == NULL) {
-        wia_log(WIA_WARNING, "Got exception from callback download->end_of_page_cb !");
+        WIA_WARNING("Pyinsane: WARNING: Got exception from callback download->end_of_page_cb !");
     }
     Py_XDECREF(res);
-
-    wia_log_set_pythread_state(download->mutex, &download->_save);
     download->_save = PyEval_SaveThread();
 
     ReleaseMutex(download->mutex);
@@ -787,19 +704,14 @@ static void end_of_scan_wrapper(void *cb_data)
     struct download *download = (struct download *)cb_data;
     PyObject *res;
 
-    wia_log(WIA_DEBUG, "end_of_scan_wrapper()");
-
     WaitForSingleObject(download->mutex, 0);
 
     PyEval_RestoreThread(download->_save);
-    wia_log_set_pythread_state(download->mutex, NULL);
-
     res = PyEval_CallObject(download->end_of_scan_cb, NULL);
     if (res == NULL) {
-        wia_log(WIA_WARNING, "Got exception from callback download->end_of_scan_cb !");
+        WIA_WARNING("Pyinsane: WARNING: Got exception from callback download->end_of_scan_cb !");
     }
     Py_XDECREF(res);
-    wia_log_set_pythread_state(download->mutex, &download->_save);
     download->_save = PyEval_SaveThread();
 
     ReleaseMutex(download->mutex);
@@ -812,42 +724,41 @@ static PyObject *download(PyObject *, PyObject *args)
     struct download dl_data = { 0 };
     struct wia_scan scan;
     HRESULT hr;
-    IWiaItem2 *src;
+    struct wia_source *src;
+
+    if (!PyEval_ThreadsInitialized()) {
+        WIA_WARNING("Python thread not yet initialized ?!");
+        PyEval_InitThreads();
+    }
 
     if (!PyArg_ParseTuple(args, "OOOOy*",
                 &capsule,
                 &dl_data.get_data_cb, &dl_data.end_of_page_cb, &dl_data.end_of_scan_cb,
                 &dl_data.buffer
             )) {
-        wia_log(WIA_WARNING, "download(): Invalid args");
-        Py_RETURN_FALSE;
+        WIA_WARNING("Pyinsane: WARNING: download(): Invalid args");
+        return NULL;
     }
 
-    wia_log(WIA_DEBUG, "download()");
-
-    /* With most scanners, scan is run on a source.
-     * With Epson Workforce ES-300W, it is run on the device itself.
-     */
-    src = capsule2item(capsule);
+    src = (struct wia_source *)PyCapsule_GetPointer(capsule, WIA_PYCAPSULE_SRC_NAME);
     if (src == NULL) {
-        wia_log(WIA_WARNING, "wrong param type. Expected a scan source");
-        Py_RETURN_FALSE;
+        WIA_WARNING("Pyinsane: WARNING: wrong param type. Expected a scan source");
+        return NULL;
     }
 
     if (!PyCallable_Check(dl_data.get_data_cb)
             || !PyCallable_Check(dl_data.end_of_page_cb)
             || !PyCallable_Check(dl_data.end_of_scan_cb)) {
-        wia_log(WIA_WARNING, "download(): wrong param type. Expected callback(s)");
-        Py_RETURN_FALSE;
+        WIA_WARNING("Pyinsane: WARNING: download(): wrong param type. Expected callback(s)");
+        Py_RETURN_NONE;
     }
 
     dl_data.mutex = CreateMutex(NULL, FALSE, NULL);
 
-    hr = src->QueryInterface(IID_IWiaTransfer, (void**)&scan.transfer);
+    hr = src->source->QueryInterface(IID_IWiaTransfer, (void**)&scan.transfer);
     if (FAILED(hr)) {
-        wia_log(WIA_WARNING, "source->QueryInterface(WiaTransfer) failed");
-        wia_log_hresult(WIA_WARNING, hr);
-        Py_RETURN_FALSE;
+        WIA_WARNING("source->QueryInterface(WiaTransfer) failed");
+        Py_RETURN_NONE;
     }
 
     scan.callbacks = new PyinsaneWiaTransferCallback(
@@ -855,42 +766,27 @@ static PyObject *download(PyObject *, PyObject *args)
         &dl_data
     );
 
-    WaitForSingleObject(dl_data.mutex, 0);
     dl_data._save = PyEval_SaveThread();
-    wia_log_set_pythread_state(dl_data.mutex, &dl_data._save);
-    ReleaseMutex(dl_data.mutex);
-
     hr = scan.transfer->Download(0, scan.callbacks);
 
     if (hr == WIA_ERROR_PAPER_EMPTY) {
         end_of_scan_wrapper((void *)&dl_data);
-
-        WaitForSingleObject(dl_data.mutex, 0);
-        wia_log_set_pythread_state(NULL, NULL);
         PyEval_RestoreThread(dl_data._save);
-        ReleaseMutex(dl_data.mutex);
-        wia_log(WIA_INFO, "No more paper");
         Py_RETURN_NONE;
     } else if (FAILED(hr)) {
-        WaitForSingleObject(dl_data.mutex, 0);
-        wia_log_set_pythread_state(NULL, NULL);
         PyEval_RestoreThread(dl_data._save);
-        ReleaseMutex(dl_data.mutex);
+        _com_error err(hr);
+        LPCTSTR errMsg = err.ErrorMessage();
 
-        wia_log(WIA_WARNING, "source->transfer->Download() failed");
-        wia_log_hresult(WIA_WARNING, hr);
+        WIA_WARNING("Pyinsane: WARNING: source->transfer->Download() failed");
+
+        std::cerr << "Pyinsane: WARNING: source->transfer->Download() failed: " << hr << " ; " << errMsg << std::endl;
 
         scan.transfer->Release();
-        Py_RETURN_FALSE;
     } else {
-        WaitForSingleObject(dl_data.mutex, 0);
-        wia_log_set_pythread_state(NULL, NULL);
         PyEval_RestoreThread(dl_data._save);
-        ReleaseMutex(dl_data.mutex);
     }
-
-    // TODO(Jflesch): destroy mutex ?
-    Py_RETURN_TRUE;
+    Py_RETURN_NONE;
 }
 
 static PyObject *exit(PyObject *, PyObject* args)
@@ -899,8 +795,6 @@ static PyObject *exit(PyObject *, PyObject* args)
         return NULL;
     }
 
-    wia_log(WIA_DEBUG, "exit()");
-
     CoUninitialize();
 
     Py_RETURN_NONE;
@@ -908,16 +802,15 @@ static PyObject *exit(PyObject *, PyObject* args)
 
 
 static PyMethodDef rawapi_methods[] = {
-    {"download", download, METH_VARARGS, NULL},
-    {"exit", exit, METH_VARARGS, NULL},
-    {"get_constraints", get_constraints, METH_VARARGS, NULL},
+    {"init", init, METH_VARARGS, NULL},
     {"get_devices", get_devices, METH_VARARGS, NULL},
     {"get_properties", get_properties, METH_VARARGS, NULL},
+    {"get_constraints", get_constraints, METH_VARARGS, NULL},
     {"get_sources", get_sources, METH_VARARGS, NULL},
-    {"init", init, METH_VARARGS, NULL},
     {"open", open_device, METH_VARARGS, NULL},
-    {"register_logger", register_logger, METH_VARARGS, NULL},
+    {"download", download, METH_VARARGS, NULL},
     {"set_property", set_property, METH_VARARGS, NULL},
+    {"exit", exit, METH_VARARGS, NULL},
     {NULL, NULL, 0, NULL},
 };
 
